/*
 * NeEM - Network-friendly Epidemic Multicast
 * Copyright (c) 2005-2006, University of Minho
 * All rights reserved.
 *
 * Contributors:
 *  - Pedro Santos <psantos@gmail.com>
 *  - Jose Orlando Pereira <jop@di.uminho.pt>
 * 
 * Partially funded by FCT, project P-SON (POSC/EIA/60941/2004).
 * See http://pson.lsd.di.uminho.pt/ for more information.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *  - Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 * 
 *  - Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 * 
 *  - Neither the name of the University of Minho nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package gsd.impl;

import java.nio.ByteBuffer;
import java.util.UUID;

/**
 * UUID manipulation utilities.
 */
public abstract class UUIDs {
	private UUIDs() {}
	
	/**
	 * Write an UUID to a ByteBuffer.
     * @param uuid The uuid to be written.
     * @return The Buffer with the uuid written into.
     */
    public static ByteBuffer writeUUIDToBuffer(UUID uuid) {
        ByteBuffer uuid_bytes = ByteBuffer.allocate(16);
        uuid_bytes.putLong(uuid.getMostSignificantBits());
        uuid_bytes.putLong(uuid.getLeastSignificantBits());
        return uuid_bytes;
    }
    
    /** Read an UUID from an array of ByteBuffers into an UUID.
     * @param msg The buffer from which to read the UUID from.
     * @return The address read.
     */
    public static UUID readUUIDFromBuffer(ByteBuffer[] msg) {
    	ByteBuffer tmp = Buffers.sliceCompact(msg, 16); 
        long msb = tmp.getLong();
        long lsb = tmp.getLong();
        return new UUID(msb, lsb);
    }

    public static void writeUUIDToBuffer(ByteBuffer buf, UUID uuid) {
        buf.putLong(uuid.getMostSignificantBits());
        buf.putLong(uuid.getLeastSignificantBits());
    }
    
    /** Read an UUID from an array of ByteBuffers into an UUID.
     * @param msg The buffer from which to read the UUID from.
     * @return The address read.
     */
    public static UUID readUUIDFromBuffer(ByteBuffer buf) { 
        long msb = buf.getLong();
        long lsb = buf.getLong();
        return new UUID(msb, lsb);
    }
}

